/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/fundingaccount/ReplaceFundingAccountHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/FundingAccountApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'), require('../../swagger/model/ReplaceFundingAccountRequest'));
    }
}(this, function (logger, ApiClient, RestConstants, FundingAccountApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum, ReplaceFundingAccountRequest) {
    'use strict';


    /**
     * The ReplaceFundingAccountHandler model module.
     * @module handler/fundingaccount/ReplaceFundingAccountHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>ReplaceFundingAccountHandler</code>.
     * @alias module:handler/fundingaccount/ReplaceFundingAccountHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (fundingAccount, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(fundingAccount);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(fundingAccount, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, fundingAccount);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (fundingAccount) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, fundingAccount);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.PATH_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.PATH_PARAM_ID_FUNDING_ACCOUNT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.PATH_PARAM_ID_FUNDING_ACCOUNT, AppErrorCodes.ID_FUNDING_ACCOUNT_INVALID, ValidationUtil.isValidId);

        this.validateFundingAccount(fundingAccount, exception);
        ValidationUtil.validateInvocationContext(fundingAccount[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.prototype.validateFundingAccount = function (fundingAccount, exception) {

        ValidationUtil.validate(fundingAccount[HttpParamConstants.ACCOUNT_HOLDER_NAME], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.ACCOUNT_HOLDER_NAME, AppErrorCodes.FUNDING_ACCOUNT_HOLDER_NAME_INVALID, ValidationUtil.isValidAccountHolderName);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.NICKNAME], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.NICKNAME, AppErrorCodes.FUNDING_ACCOUNT_NICKNAME_INVALID, ValidationUtil.isValidNickName);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.ACCOUNT_NUMBER], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.ACCOUNT_NUMBER, AppErrorCodes.FUNDING_ACCOUNT_NUMBER_INVALID, ValidationUtil.isValidFundingAccountNumber);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.ADDRESS], DataTypeEnum.Object, MandatoryEnum.no, exception,
            HttpParamConstants.ADDRESS, AppErrorCodes.SYSTEM_ERROR, ValidationUtil.validateFundingAccountAddress);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.ABA_ROUTING_NUMBER], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.ABA_ROUTING_NUMBER, AppErrorCodes.FORM_PARAM_FUNDING_ACCOUNT_ABA_ROUTING_NUMBER_INVALID, ValidationUtil.isValidABARoutingNumber);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.EXPIRY_DATE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.EXPIRY_DATE, AppErrorCodes.EXPIRY_DATE_INVALID, ValidationUtil.isValidExpiryDate);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.ACCOUNT_HOLDER_TYPE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.ACCOUNT_HOLDER_TYPE, AppErrorCodes.ACCOUNT_HOLDER_TYPE_INVALID, ValidationUtil.isValidFundingAccountHolderType);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.ACCOUNT_SUBTYPE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.ACCOUNT_SUBTYPE, AppErrorCodes.FUNDING_ACCOUNT_SUBTYPE_INVALID, ValidationUtil.isValidFundingAccountSubtype);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.CARD_CVV_NUMBER], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.CARD_CVV_NUMBER, AppErrorCodes.CARD_CVV_NUMBER_INVALID, ValidationUtil.isValidCardCVVNumber);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.COMMENTS], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.COMMENTS, AppErrorCodes.COMMENTS_INVALID, ValidationUtil.isValidComments);
    };

    exports.prototype.getOptions = function (fundingAccount) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(fundingAccount[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES])
        };
    };

    exports.prototype.callAPI = function (fundingAccount, callback) {
        var invocationContext = fundingAccount[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var replaceFundingAccountRequest = ReplaceFundingAccountRequest.constructFromObject(fundingAccount);

        var opts = this.getOptions(fundingAccount);

        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], fundingAccount[HttpParamConstants.API_END_POINT]);
        var fundingAccountApi = new FundingAccountApi(apiClient);
        var request = fundingAccountApi.replaceFundingAccount(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            fundingAccount[HttpParamConstants.PATH_PARAM_ID_FUNDING_ACCOUNT], fundingAccount[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], replaceFundingAccountRequest, opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));


