/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/customer/RetrieveFundingAccountsHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/FundingAccountApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, FundingAccountApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The RetrieveFundingAccountsHandler model module.
     * @module handler/customer/RetrieveFundingAccountsHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>RetrieveFundingAccountsHandler</code>.
     * @alias module:handler/customer/RetrieveFundingAccountsHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (fundingAccounts, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(fundingAccounts);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(fundingAccounts, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, fundingAccounts);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (fundingAccounts) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, fundingAccounts);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.true, exception,
            HttpParamConstants.ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.PAGE_SIZE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.PAGE_SIZE, AppErrorCodes.PAGE_SIZE_INVALID, ValidationUtil.isValidPageSize);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.ACCOUNT_NUMBER], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.ACCOUNT_NUMBER, AppErrorCodes.FORM_PARAM_FUNDING_ACCOUNT_NUMBER_INVALID, ValidationUtil.isValidFundingAccountNumber);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.STATUS], DataTypeEnum.Array, MandatoryEnum.false, exception,
            HttpParamConstants.STATUS, AppErrorCodes.FORM_PARAM_FUNDING_ACCOUNT_STATUS_INVALID, ValidationUtil.isValidFundingAccountStatuses);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.ABA_ROUTING_NUMBER], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.ABA_ROUTING_NUMBER, AppErrorCodes.FORM_PARAM_FUNDING_ACCOUNT_ABA_ROUTING_NUMBER_INVALID, ValidationUtil.isValidABARoutingNumber);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.ACCOUNT_TYPE], DataTypeEnum.Array, MandatoryEnum.false, exception,
            HttpParamConstants.ACCOUNT_TYPE, AppErrorCodes.FORM_PARAM_FUNDING_ACCOUNT_TYPE_INVALID, ValidationUtil.isValidFundingAccountTypes);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.ACCOUNT_SUBTYPE], DataTypeEnum.Array, MandatoryEnum.false, exception,
            HttpParamConstants.ACCOUNT_SUBTYPE, AppErrorCodes.FORM_PARAM_FUNDING_ACCOUNT_SUBTYPE_INVALID, ValidationUtil.isValidFundingAccountSubtypes);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.QUERY_ID], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_ID, AppErrorCodes.QUERY_ID_INVALID, ValidationUtil.isValidQueryId);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.FROM_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.FROM_INDEX, AppErrorCodes.FROM_INDEX_INVALID, ValidationUtil.isValidFromIndex);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.TO_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.TO_INDEX, AppErrorCodes.TO_INDEX_INVALID, ValidationUtil.isValidToIndex);

        ValidationUtil.validateInvocationContext(fundingAccounts[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(fundingAccounts[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.prototype.getOptions = function (fundingAccounts) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(fundingAccounts[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES]),
            'pageSize': fundingAccounts[HttpParamConstants.PAGE_SIZE],
            'accountNumber': fundingAccounts[HttpParamConstants.ACCOUNT_NUMBER],
            'status': fundingAccounts[HttpParamConstants.STATUS],
            'abaRoutingNumber': fundingAccounts[HttpParamConstants.ABA_ROUTING_NUMBER],
            'accountType': fundingAccounts[HttpParamConstants.ACCOUNT_TYPE],
            'accountSubtype': fundingAccounts[HttpParamConstants.ACCOUNT_SUBTYPE]
        };
    };

    exports.prototype.getPageOptions = function (fundingAccounts) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(fundingAccounts[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES]),
            'pageSize': fundingAccounts[HttpParamConstants.PAGE_SIZE],
            'fromIndex': fundingAccounts[HttpParamConstants.FROM_INDEX],
            'toIndex': fundingAccounts[HttpParamConstants.TO_INDEX]
        };
    };

    exports.prototype.callAPI = function (fundingAccounts, callback) {
        var invocationContext = fundingAccounts[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];
        var queryId = fundingAccounts[HttpParamConstants.QUERY_ID];
        var ID_CUSTOMER = fundingAccounts[HttpParamConstants.PATH_PARAM_ID_CUSTOMER];

        var opts;
        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], fundingAccounts[HttpParamConstants.API_END_POINT]);
        var fundingAccountApi = new FundingAccountApi(apiClient);
        var request;

        if (ValidationUtil.isNonEmptyString(fundingAccounts[HttpParamConstants.QUERY_ID])) {
            opts = this.getPageOptions(fundingAccounts);
            request = fundingAccountApi.retrieveFundingAccountsPage(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType, ID_CUSTOMER, queryId, opts, callback);
        } else {
            opts = this.getOptions(fundingAccounts);
            request = fundingAccountApi.retrieveFundingAccounts(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType, ID_CUSTOMER, opts, callback);
        }

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));
