/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/payment/DeletePaymentHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/PaymentApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'), require('../../swagger/model/DeletePaymentRequest'));
    }
}(this, function (logger, ApiClient, RestConstants, PaymentApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum, DeletePaymentRequest) {
    'use strict';


    /**
     * The DeletePaymentHandler model module.
     * @module handler/payment/DeletePaymentHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>DeletePaymentHandler</code>.
     * @alias module:handler/payment/DeletePaymentHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (payment, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(payment);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(payment, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, payment);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (payment) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, payment);

        ValidationUtil.validate(payment[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.PATH_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(payment[HttpParamConstants.PATH_PARAM_ID_PAYMENT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.PATH_PARAM_ID_PAYMENT, AppErrorCodes.ID_PAYMENT_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validateInvocationContext(payment[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(payment[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.prototype.getOptions = function (payment) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(payment[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES])
        };
    };

    exports.prototype.callAPI = function (payment, callback) {
        var invocationContext = payment[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var deletePaymentRequest = DeletePaymentRequest.constructFromObject(payment);

        var opts = this.getOptions(payment);

        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], payment[HttpParamConstants.API_END_POINT]);
        var paymentApi = new PaymentApi(apiClient);
        var request = paymentApi.deletePayment(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            payment[HttpParamConstants.PATH_PARAM_ID_PAYMENT], payment[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], deletePaymentRequest, opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));


