/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentSetupsapi-client/handler/customer/RetrievePaymentSetupsHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/PaymentSetupApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, PaymentSetupApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The RetrievePaymentSetupsHandler model module.
     * @module handler/customer/RetrievePaymentSetupsHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>RetrievePaymentSetupsHandler</code>.
     * @alias module:handler/customer/RetrievePaymentSetupsHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (paymentSetups, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(paymentSetups);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(paymentSetups, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, paymentSetups);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (paymentSetups) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, paymentSetups);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.PAGE_SIZE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.PAGE_SIZE, AppErrorCodes.PAGE_SIZE_INVALID, ValidationUtil.isValidPageSize);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.QUERY_PARAM_ID_FUNDING_ACCOUNT], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_PARAM_ID_FUNDING_ACCOUNT, AppErrorCodes.ID_FUNDING_ACCOUNT_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER_ACCOUNT], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_PARAM_ID_CUSTOMER_ACCOUNT, AppErrorCodes.ID_CUSTOMER_ACCOUNT_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.CONFIRMATION_NUMBER], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.CONFIRMATION_NUMBER, AppErrorCodes.PAYMENT_SETUP_CONFIRMATION_NUMBER_INVALID, ValidationUtil.isValidConfirmationNumber);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.STATUS], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.STATUS, AppErrorCodes.QUERY_PARAM_PAYMENT_SETUP_STATUS_INVALID, ValidationUtil.isValidPaymentSetupStatuses);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.PAYMENT_SETUP_SCHEDULE_TYPE], DataTypeEnum.Array, MandatoryEnum.false, exception,
            HttpParamConstants.PAYMENT_SETUP_SCHEDULE_TYPE, AppErrorCodes.PAYMENT_SETUP_SCHEDULE_TYPE_INVALID, ValidationUtil.isValidPaymentSetupScheduleTypes);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.FROM_DATE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.FROM_DATE, AppErrorCodes.PAYMENT_SETUP_FROM_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.TO_DATE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.TO_DATE, AppErrorCodes.PAYMENT_SETUP_TO_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.QUERY_ID], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_ID, AppErrorCodes.QUERY_ID_INVALID, ValidationUtil.isValidQueryId);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.FROM_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.FROM_INDEX, AppErrorCodes.FROM_INDEX_INVALID, ValidationUtil.isValidFromIndex);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.TO_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.TO_INDEX, AppErrorCodes.TO_INDEX_INVALID, ValidationUtil.isValidToIndex);

        ValidationUtil.validateInvocationContext(paymentSetups[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(paymentSetups[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.prototype.getOptions = function (paymentSetups) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(paymentSetups[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES]),
            'idCustomer': paymentSetups[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER],
            'idFundingAccount': paymentSetups[HttpParamConstants.QUERY_PARAM_ID_FUNDING_ACCOUNT],
            'idCustomerAccount': paymentSetups[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER_ACCOUNT],
            'confirmationNumber': paymentSetups[HttpParamConstants.CONFIRMATION_NUMBER],
            'status': paymentSetups[HttpParamConstants.STATUS],
            'paymentSetupScheduleType': paymentSetups[HttpParamConstants.PAYMENT_SETUP_SCHEDULE_TYPE],
            'fromDate': paymentSetups[HttpParamConstants.FROM_DATE],
            'toDate': paymentSetups[HttpParamConstants.TO_DATE],
            'pageSize': paymentSetups[HttpParamConstants.PAGE_SIZE],
            'queryId': paymentSetups[HttpParamConstants.QUERY_ID],
            'fromIndex': paymentSetups[HttpParamConstants.FROM_INDEX],
            'toIndex': paymentSetups[HttpParamConstants.TO_INDEX]
        };
    };

    exports.prototype.callAPI = function (paymentSetups, callback) {
        var _this = this;
        var invocationContext = paymentSetups[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var opts = _this.getOptions(paymentSetups);
        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], paymentSetups[HttpParamConstants.API_END_POINT]);
        var paymentSetupApi = new PaymentSetupApi(apiClient);
        var request = paymentSetupApi.retrievePaymentSetups(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType, opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));
