/**
 * Created by Alacriti Inc on 10/7/19.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/paymentsetup/UpdatePaymentSetupHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/PaymentSetupApi'), require('./../../util/CommonUtil'), require('../../swagger/model/UpdatePaymentSetupRequest'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, PaymentSetupApi, CommonUtil, UpdatePaymentSetupRequest, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The CreatePaymentSetupHandler model module.
     * @module handler/paymentsetup/UpdatePaymentSetupHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>CreatePaymentSetupHandler</code>.
     * @alias module:handler/paymentsetup/UpdatePaymentSetupHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (paymentSetup, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(paymentSetup);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(paymentSetup, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, paymentSetup);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (paymentSetup) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, paymentSetup);

        ValidationUtil.validate(paymentSetup[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.yes,
            exception, HttpParamConstants.PATH_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(paymentSetup[HttpParamConstants.PATH_PARAM_ID_PAYMENT_SETUP], DataTypeEnum.String, MandatoryEnum.yes,
            exception, HttpParamConstants.PATH_PARAM_ID_PAYMENT_SETUP, AppErrorCodes.ID_PAYMENT_SETUP_INVALID, ValidationUtil.isValidId);

        this.validatePaymentSetup(paymentSetup, exception);

        ValidationUtil.validateInvocationContext(paymentSetup[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(paymentSetup[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };


    exports.prototype.validatePaymentSchedule = function (paymentSchedule, exception, segment) {

        segment += HttpParamConstants.DOT;

        ValidationUtil.validate(paymentSchedule[HttpParamConstants.PAYMENT_LIMIT_AMOUNT], DataTypeEnum.String, MandatoryEnum.no,
            exception, segment + HttpParamConstants.PAYMENT_LIMIT_AMOUNT, AppErrorCodes.PAYMENT_LIMIT_AMOUNT_INVALID, ValidationUtil.isValidAmount);

    };

    exports.prototype.validateFundingAccount = function (fundingAccount, exception, segment) {
        logger.debug('Fundingaccount: '+fundingAccount);
        segment += HttpParamConstants.DOT;
        ValidationUtil.validate(fundingAccount[HttpParamConstants.ID], DataTypeEnum.String, MandatoryEnum.no,
            exception, segment + HttpParamConstants.ID, AppErrorCodes.ID_FUNDING_ACCOUNT_INVALID, ValidationUtil.isValidId);
    };

    exports.prototype.validatePaymentSetup = function (paymentSetup, exception) {

        ValidationUtil.validate(paymentSetup[HttpParamConstants.FUNDING_ACCOUNT], DataTypeEnum.Object, MandatoryEnum.no, exception,
            HttpParamConstants.FUNDING_ACCOUNT, AppErrorCodes.ERROR_SEGMENT_MISSING, this.validateFundingAccount);

        ValidationUtil.validate(paymentSetup[HttpParamConstants.PAYMENT_SCHEDULE], DataTypeEnum.Object, MandatoryEnum.no, exception,
            HttpParamConstants.PAYMENT_SCHEDULE, AppErrorCodes.ERROR_SEGMENT_MISSING, this.validatePaymentSchedule);

        ValidationUtil.validate(paymentSetup[HttpParamConstants.COMMENTS], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.COMMENTS, AppErrorCodes.COMMENTS_INVALID, ValidationUtil.isValidComments);

    };

    exports.prototype.getOptions = function (paymentSetup) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(paymentSetup[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES])
        };
    };

    exports.prototype.callAPI = function (paymentSetup, callback) {
        var invocationContext = paymentSetup[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var updatePaymentSetupRequest = UpdatePaymentSetupRequest.constructFromObject(paymentSetup);

        var opts = this.getOptions(paymentSetup);

        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], paymentSetup[HttpParamConstants.API_END_POINT]);
        var paymentSetupApi = new PaymentSetupApi(apiClient);
        var request = paymentSetupApi.updatePaymentSetup(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            paymentSetup[HttpParamConstants.PATH_PARAM_ID_PAYMENT_SETUP],paymentSetup[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], updatePaymentSetupRequest, opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));


