/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/refund/CreateRefundHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/RefundApi'), require('./../../util/CommonUtil'), require('../../swagger/model/CreateRefundRequest'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, RefundApi, CommonUtil, CreateRefundRequest, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The CreateRefundHandler model module.
     * @module handler/refund/CreateRefundHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>CreateRefundHandler</code>.
     * @alias module:handler/refund/CreateRefundHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (refund, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(refund);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(refund, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, refund);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (refund) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, refund);
        ValidationUtil.validate(refund[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.yes,
            exception, HttpParamConstants.PATH_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(refund[HttpParamConstants.PATH_PARAM_ID_PAYMENT], DataTypeEnum.String, MandatoryEnum.yes,
            exception, HttpParamConstants.PATH_PARAM_ID_PAYMENT, AppErrorCodes.ID_PAYMENT_INVALID, ValidationUtil.isValidId);

        this.validateRefund(refund, exception);
        ValidationUtil.validateInvocationContext(refund[HttpParamConstants.INVOCATION_CONTEXT], exception);

        return exception;
    };

    exports.prototype.validateRefund = function (refund, exception) {

        ValidationUtil.validate(refund[HttpParamConstants.REFUND_REFERENCE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.REFUND_REFERENCE, AppErrorCodes.REFUND_REFERENCE_INVALID, ValidationUtil.isValidRefundReference);

        ValidationUtil.validate(refund[HttpParamConstants.REFUND_AMOUNT], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.REFUND_AMOUNT, AppErrorCodes.AMOUNT_INVALID, ValidationUtil.isValidAmount);

        ValidationUtil.validate(refund[HttpParamConstants.COMMENTS], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.COMMENTS, AppErrorCodes.COMMENTS_INVALID, ValidationUtil.isValidComments);

    };

    exports.prototype.getOptions = function (refund) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(refund[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES])
        };
    };

    exports.prototype.callAPI = function (refund, callback) {
        var invocationContext = refund[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var createRefundRequest = CreateRefundRequest.constructFromObject(refund);

        var opts = this.getOptions(refund);

        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], refund[HttpParamConstants.API_END_POINT]);
        var refundApi = new RefundApi(apiClient);
        var request = refundApi.createRefund(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            refund[HttpParamConstants.PATH_PARAM_ID_PAYMENT], refund[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], createRefundRequest, opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));


