/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/refund/RetrieveRefundsHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/RefundApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, RefundApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The RetrieveRefundsHandler model module.
     * @module handler/refund/RetrievePaymentsHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>RetrieveRefundsHandler</code>.
     * @alias module:handler/refund/RetrieveRefundsHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (refunds, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(refunds);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(refunds, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, refunds);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (refunds) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, refunds);

        ValidationUtil.validate(refunds[HttpParamConstants.PAGE_SIZE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.PAGE_SIZE, AppErrorCodes.PAGE_SIZE_INVALID, ValidationUtil.isValidPageSize);

        ValidationUtil.validate(refunds[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(refunds[HttpParamConstants.CONFIRMATION_NUMBER], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.CONFIRMATION_NUMBER, AppErrorCodes.PAYMENT_CONFIRMATION_NUMBER_INVALID, ValidationUtil.isValidConfirmationNumber);

        ValidationUtil.validate(refunds[HttpParamConstants.STATUS], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.STATUS, AppErrorCodes.QUERY_PARAM_PAYMENT_STATUS_INVALID, ValidationUtil.isValidrefundstatuses);

        ValidationUtil.validate(refunds[HttpParamConstants.FROM_DATE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.FROM_DATE, AppErrorCodes.PAYMENT_FROM_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(refunds[HttpParamConstants.TO_DATE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.TO_DATE, AppErrorCodes.PAYMENT_TO_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(refunds[HttpParamConstants.QUERY_ID], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_ID, AppErrorCodes.QUERY_ID_INVALID, ValidationUtil.isValidQueryId);

        ValidationUtil.validate(refunds[HttpParamConstants.FROM_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.FROM_INDEX, AppErrorCodes.FROM_INDEX_INVALID, ValidationUtil.isValidFromIndex);

        ValidationUtil.validate(refunds[HttpParamConstants.TO_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.TO_INDEX, AppErrorCodes.TO_INDEX_INVALID, ValidationUtil.isValidToIndex);

        ValidationUtil.validateInvocationContext(refunds[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(refunds[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.prototype.getOptions = function (refunds) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(refunds[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES]),
            'idCustomer': refunds[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER],
            'confirmationNumber': refunds[HttpParamConstants.CONFIRMATION_NUMBER],
            'status': refunds[HttpParamConstants.STATUS],
            'fromDate': refunds[HttpParamConstants.FROM_DATE],
            'toDate': refunds[HttpParamConstants.TO_DATE],
            'pageSize': refunds[HttpParamConstants.PAGE_SIZE],
            'queryId': refunds[HttpParamConstants.QUERY_ID],
            'fromIndex': refunds[HttpParamConstants.FROM_INDEX],
            'toIndex': refunds[HttpParamConstants.TO_INDEX]
        };
    };

    exports.prototype.callAPI = function (refunds, callback) {
        var _this = this;
        var invocationContext = refunds[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var opts = _this.getOptions(refunds);
        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], refunds[HttpParamConstants.API_END_POINT]);
        var refundApi = new RefundApi(apiClient);
        var request = refundApi.retrieveRefunds(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));
