/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/base/InvocationContext'), require('./Attributes'));
    }
}(this, function (logger, Attributes) {
    'use strict';


    /**
     * The InvocationContext model module.
     * @module model/base/InvocationContext
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>InvocationContext</code>.
     * @alias module:model/base/InvocationContext
     * @class
     */
    var exports = function (idempotent_request_key, secret, attributes) {
        var _this = this;
        _this['idempotent_request_key'] = idempotent_request_key;
        _this['secret'] = secret;
        _this['attributes'] = attributes;
    };

    /**
     * @private
     * Constructs a <code>InvocationContext</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest(response headers).
     * @param {module:model/base/InvocationContext} obj Optional instance to populate.
     * @return {module:model/base/InvocationContext} The populated <code>InvocationContext</code> instance.
     */
    exports.constructFromHeaders = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('channel')) {
                obj['channel'] = data['channel'];
            }
            if (data.hasOwnProperty('client_key')) {
                obj['client_key'] = data['client_key'];
            }
            if (data.hasOwnProperty('product')) {
                obj['product'] = data['product'];
            }
            if (data.hasOwnProperty('timestamp')) {
                obj['timestamp'] = data['timestamp'];
            }
            if (data.hasOwnProperty('idempotent_request_key')) {
                obj['idempotent_request_key'] = data['idempotent_request_key'];
            }
            if (data.hasOwnProperty('requestor')) {
                obj['requestor'] = data['requestor'];
            }
            if (data.hasOwnProperty('requestor_type')) {
                obj['requestor_type'] = data['requestor_type'];
            }
            if (data.hasOwnProperty('X-OPAY-Headers'.toLowerCase())) {
                obj['attributes'] = Attributes.constructFromXOPAYHeaders(data['X-OPAY-Headers'.toLowerCase()]);
            }
            if (data.hasOwnProperty('request_uuid')) {
                obj['request_uuid'] = data['request_uuid'];
            }
            if (data.hasOwnProperty('response_codes')) {
                obj['response_codes'] = data['response_codes'];
            }
            if (data.hasOwnProperty('response_text')) {
                obj['response_text'] = data['response_text'];
            }
            if (data.hasOwnProperty('secret')) {
                obj['secret'] = data['secret'];
            }
        }
        return obj;
    };


    /**
     * Constructs a <code>InvocationContext</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/base/InvocationContext} obj Optional instance to populate.
     * @return {module:model/base/InvocationContext} The populated <code>InvocationContext</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('channel')) {
                obj['channel'] = data['channel'];
            }
            if (data.hasOwnProperty('client_key')) {
                obj['client_key'] = data['client_key'];
            }
            if (data.hasOwnProperty('product')) {
                obj['product'] = data['product'];
            }
            if (data.hasOwnProperty('timestamp')) {
                obj['timestamp'] = data['timestamp'];
            }
            if (data.hasOwnProperty('idempotent_request_key')) {
                obj['idempotent_request_key'] = data['idempotent_request_key'];
            }
            if (data.hasOwnProperty('requestor')) {
                obj['requestor'] = data['requestor'];
            }
            if (data.hasOwnProperty('requestor_type')) {
                obj['requestor_type'] = data['requestor_type'];
            }
            if (data.hasOwnProperty('attributes')) {
                obj['attributes'] = Attributes.constructFromObject(data['attributes']);
            }
            if (data.hasOwnProperty('request_uuid')) {
                obj['request_uuid'] = data['request_uuid'];
            }
            if (data.hasOwnProperty('response_codes')) {
                obj['response_codes'] = data['response_codes'];
            }
            if (data.hasOwnProperty('response_text')) {
                obj['response_text'] = data['response_text'];
            }
            if (data.hasOwnProperty('secret')) {
                obj['secret'] = data['secret'];
            }
        }
        return obj;
    };

    /**
     * @member {String} channel
     */
    exports.prototype['channel'] = undefined;
    /**
     * @member {String} client_key
     */
    exports.prototype['client_key'] = undefined;
    /**
     * @member {String} product
     */
    exports.prototype['product'] = undefined;
    /**
     * @member {String} timestamp
     */
    exports.prototype['timestamp'] = undefined;
    /**
     * @member {String} idempotent_request_key
     */
    exports.prototype['idempotent_request_key'] = undefined;
    /**
     * @member {String} requestor
     */
    exports.prototype['requestor'] = undefined;
    /**
     * @member {String} requestor_type
     */
    exports.prototype['requestor_type'] = undefined;
    /**
     * @member {module:model/base/Attributes} attributes
     */
    exports.prototype['attributes'] = undefined;
    /**
     * @member {String} request_uuid
     */
    exports.prototype['request_uuid'] = undefined;
    /**
     * @member {String} response_codes
     */
    exports.prototype['response_codes'] = undefined;
    /**
     * @member {String} response_text
     */
    exports.prototype['response_text'] = undefined;
    /**
     * @member {String} secret
     */
    exports.prototype['secret'] = undefined;

    return exports;
}));