/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/base/MultiMap'));
    }
}(this, function (logger) {
    'use strict';


    /**
     * The MultiMap model module.
     * @module model/base/MultiMap
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>MultiMap</code>.
     * @alias module:model/base/MultiMap
     * @class
     */
    var exports = function () {
        var _this = this;
        _this['map'] = {};
    };

    /**
     * @private
     * @member {Object} map
     */
    exports.prototype['map'] = undefined;

    /**
     * @param {String} key
     * @param {String} value
     */
    exports.prototype.put = function (key, value) {
        var _this = this;
        if (typeof key === 'string' && key !== '' && typeof value === 'string' && value !== '') {

            if (!Array.isArray(_this['map'][key])) {
                _this['map'][key] = [];
            }

            if (_this['map'][key].indexOf(value) === -1)
                _this['map'][key].push(value);
        }
    };

    /**
     * @param {String} key
     * @return {Array} values of the key provided.
     */
    exports.prototype.get = function (key) {
        var _this = this;
        return _this['map'][key];
    };

    /**
     * @param {String} key
     * @return {Boolean}
     */
    exports.prototype.has = function (key) {
        var _this = this;
        return _this['map'].hasOwnProperty(key);
    };

    return exports;
}));