/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/customeraccount/CustomerAccount'), require('./../base/Address'), require('./../base/Base'), require('./../base/InvocationContext'), require('./../../enum/InvocationModeEnum'), require('./../../constant/RestConstants'), require('./../../enum/ProductEnum'), require('./../base/Attributes'), require('./../../handler/customeraccount/GetCustomerAccountHandler'), require('./../../util/CommonUtil'), require('./../../util/ValidationUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ResponseWrapper'));
    }
}(this, function (logger, Address, Base, InvocationContext, InvocationModeEnum, RestConstants, ProductEnum, Attributes, GetCustomerAccountHandler, CommonUtil, ValidationUtil, OrbipayApiException, ResponseWrapper) {
    'use strict';


    /**
     * The CustomerAccount model module.
     * @module model/customeraccount/CustomerAccount
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>CustomerAccount</code>.
     * @alias module:model/customeraccount/CustomerAccount
     * @class
     * @param ID_CUSTOMER_ACCOUNT {String} Id is a unique identifier assigned to the customer account in Orbipay.
     */
    var exports = function (ID_CUSTOMER_ACCOUNT) {
        var _this = this;
        Base.call(_this);
        _this['ID_CUSTOMER_ACCOUNT'] = ID_CUSTOMER_ACCOUNT;
        _this['id'] = ID_CUSTOMER_ACCOUNT;
    };

    /**
     * @private
     * Constructs a <code>CustomerAccount</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/customeraccount/CustomerAccount} obj Optional instance to populate.
     * @return {module:model/customeraccount/CustomerAccount} The populated <code>CustomerAccount</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('account_holder_name')) {
                obj['account_holder_name'] = data['account_holder_name'];
            }
            if (data.hasOwnProperty('nickname')) {
                obj['nickname'] = data['nickname'];
            }
            if (data.hasOwnProperty('address')) {
                obj['address'] = Address.constructFromObject(data['address']);
            }
            if (data.hasOwnProperty('customer_account_reference')) {
                obj['customer_account_reference'] = data['customer_account_reference'];
            }
            if (data.hasOwnProperty('account_number')) {
                obj['account_number'] = data['account_number'];
            }
            if (data.hasOwnProperty('current_balance')) {
                obj['current_balance'] = data['current_balance'];
            }
            if (data.hasOwnProperty('current_statement_balance')) {
                obj['current_statement_balance'] = data['current_statement_balance'];
            }
            if (data.hasOwnProperty('minimum_payment_due')) {
                obj['minimum_payment_due'] = data['minimum_payment_due'];
            }
            if (data.hasOwnProperty('past_amount_due')) {
                obj['past_amount_due'] = data['past_amount_due'];
            }
            if (data.hasOwnProperty('payment_due_date')) {
                obj['payment_due_date'] = data['payment_due_date'];
            }
            if (data.hasOwnProperty('statement_date')) {
                obj['statement_date'] = data['statement_date'];
            }
            if (data.hasOwnProperty('id')) {
                obj['id'] = data['id'];
            }
            if (data.hasOwnProperty('url')) {
                obj['url'] = data['url'];
            }
            if (data.hasOwnProperty('status')) {
                obj['status'] = data['status'];
            }
            if (data.hasOwnProperty('custom_fields')) {
                obj['custom_fields'] = data['custom_fields'];
            }
        }
        return obj;
    };

    exports.prototype = Object.create(Base.prototype);

    exports.prototype.constructor = exports;

    /**
     * The name as specified on the account.
     * @member {String} account_holder_name
     */
    exports.prototype['account_holder_name'] = undefined;
    /**
     * The nickname by which a customer might want to identify the account.
     * @member {String} nickname
     */
    exports.prototype['nickname'] = undefined;
    /**
     * @member {module:model/base/Address} address
     */
    exports.prototype['address'] = undefined;
    /**
     * The unique identifier in the client system for the customer account.
     * @member {String} customer_account_reference
     */
    exports.prototype['customer_account_reference'] = undefined;
    /**
     * The number or reference that a customer uses to identify the customer's account.
     * @member {String} account_number
     */
    exports.prototype['account_number'] = undefined;
    /**
     * The current balance on the customer's account with the biller.
     * @member {String} current_balance
     */
    exports.prototype['current_balance'] = undefined;
    /**
     * The balance on the customer’s account with the biller as reported on the last statement.
     * @member {String} current_statement_balance
     */
    exports.prototype['current_statement_balance'] = undefined;
    /**
     * The minimum payment amount, as specified by the biller, that needs to be made against the customer’s account.
     * @member {String} minimum_payment_due
     */
    exports.prototype['minimum_payment_due'] = undefined;
    /**
     * The amount that is due from the past bills/statements on the customer’s account with the biller.
     * @member {String} past_amount_due
     */
    exports.prototype['past_amount_due'] = undefined;
    /**
     * The date by which the customer needs to clear the dues against the customer's account with the biller. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.
     * @member {String} payment_due_date
     */
    exports.prototype['payment_due_date'] = undefined;
    /**
     * The date on which the biller generates the statement for the customer's account. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.
     * @member {String} statement_date
     */
    exports.prototype['statement_date'] = undefined;
    /**
     * Id is a unique identifier assigned to the customer account in Orbipay.
     * @member {String} id
     */
    exports.prototype['id'] = undefined;
    /**
     * This URL fetches the Account details.
     * @member {String} url
     */
    exports.prototype['url'] = undefined;
    /**
     * The status of the customer's account in Orbipay.
     * @member {module:model/customeraccount/CustomerAccount.StatusEnum} status
     */
    exports.prototype['status'] = undefined;
    /**
     * The additional information or meta-information that Orbipay can accept, maintain and transmit back to the client. The custom fields need to be configured with Orbipay before they can be accepted. Orbipay would reject custom fields that are not pre-configured. Please contact support@billerpayments.com for more information on configuring and using custom fields.
     * @member {Object} custom_fields
     */
    exports.prototype['custom_fields'] = undefined;

    /**
     * @param {String} ID_CUSTOMER
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.forCustomer = function (ID_CUSTOMER) {
        var _this = this;
        _this['ID_CUSTOMER'] = ID_CUSTOMER;
        return _this;
    };

    /**
     * @param {String} account_holder_name
     * @param {String} nickname
     * @param {String} account_number
     * @param {String} minimum_payment_due
     * @param {String} past_amount_due
     * @param {String} payment_due_date
     * @param {String} statement_date
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.withDetails = function (account_holder_name, nickname, account_number, minimum_payment_due, past_amount_due, payment_due_date, statement_date) {
        var _this = this;
        _this['account_holder_name'] = account_holder_name;
        _this['nickname'] = nickname;
        _this['account_number'] = account_number;
        _this['minimum_payment_due'] = minimum_payment_due;
        _this['past_amount_due'] = past_amount_due;
        _this['payment_due_date'] = payment_due_date;
        _this['statement_date'] = statement_date;
        return _this;
    };

    /**
     * @param {String} current_balance
     * @param {String} current_statement_balance
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.withBalances = function (current_balance, current_statement_balance) {
        var _this = this;
        _this['current_balance'] = current_balance;
        _this['current_statement_balance'] = current_statement_balance;
        return _this;
    };

    /**
     * @param {String} customer_account_reference
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.withReference = function (customer_account_reference) {
        var _this = this;
        _this['customer_account_reference'] = customer_account_reference;
        return _this;
    };

    /**
     * @param {String} status. Use {module:enum/CustomerAccountStatusEnum} to get values.
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.withStatus = function (status) {
        var _this = this;
        this['status'] = status;
        return _this;
    };

    /**
     * @param {module:model/base/Address} address.
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.withAddress = function (address) {
        var _this = this;
        _this['address'] = Address.constructFromObject(address);
        return _this;
    };

    /**
     * @param {Object} custom_fields
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.withCustomFields = function (custom_fields) {
        var _this = this;
        _this['custom_fields'] = custom_fields;
        return _this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.retrievedBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };

    /**
     * @param {String} channel
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.videChannel = function (channel) {
        var _this = this;
        Base.prototype.videChannel.call(_this, channel);
        return _this;
    };

    /**
     * @param {String} client_key
     * @return module:model/customeraccount/CustomerAccount
     */
    exports.prototype.forClient = function (client_key) {
        var _this = this;
        Base.prototype.forClient.call(_this, client_key);
        return _this;
    };

    /**
     * Callback function to receive the result of the operation.
     * @callback module:model/customeraccount/CustomerAccounts~callApiCallback
     * @param {module:exception/OrbipayApiException} exception Error Response, if any.
     * @param {module:model/customeraccount/CustomerAccounts} data The data returned by the service call.
     */


    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/customeraccount/CustomerAccounts~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.get = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;
        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new GetCustomerAccountHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    return exports;
}));
