/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/error/Error'), require('./../../constant/RestConstants'));
    }
}(this, function (logger, RestConstants) {
    'use strict';


    /**
     * The Error model module.
     * @module model/error/Error
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>Error</code>.
     * @alias module:model/exception/Error
     * @class
     * @param code {String}
     * @param message {String}
     * @param field {String}
     */
    var exports = function (code, message, field) {
        var _this = this;
        _this['code'] = code;
        _this['message'] = message;
        _this['field'] = field;
    };

    /**
     * @private
     * Constructs a <code>Error</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/error/Error} obj Optional instance to populate.
     * @return {module:model/error/Error} The populated <code>Error</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('code')) {
                obj['code'] = data['code'];
            }
            if (data.hasOwnProperty('message')) {
                obj['message'] = data['message'];
            }
            if (data.hasOwnProperty('field')) {
                obj['field'] = data['field'];
            }
        }
        return obj;
    };

    /**
     * @private
     * @param {String} field Name of the field.
     * @return module:model/error/Error
     */
    exports.getFieldError = function (field) {
        var obj = obj || new exports();
        obj['code'] = RestConstants.ERROR_FIELD_CODE;
        obj['message'] = RestConstants.ERROR_FIELD_MESSAGE;
        obj['field'] = field;
        return obj;
    };

    /**
     * @private
     * @return module:model/error/Error
     */
    exports.getTechDiffError = function () {
        var obj = obj || new exports();
        obj['code'] = RestConstants.ERROR_TECH_DIFFICULTIES_CODE;
        obj['message'] = RestConstants.ERROR_TECH_DIFFICULTIES_MESSAGE;
        return obj;
    };

    /**
     * @member {String} code
     */
    exports.prototype['code'] = undefined;
    /**
     * @member {String} message
     */
    exports.prototype['message'] = undefined;
    /**
     * Name of the invalid parameter.
     * @member {String} field
     */
    exports.prototype['field'] = undefined;

    return exports;
}));


