/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/fundingaccount/FundingAccount'), require('./../base/Address'), require('./../base/Base'), require('./../base/InvocationContext'), require('./../../constant/RestConstants'), require('./../../enum/ProductEnum'), require('./../base/Attributes'), require('./../../enum/InvocationModeEnum'), require('./../../handler/fundingaccount/AddFundingAccountHandler'), require('./../../handler/fundingaccount/UpdateFundingAccountHandler'), require('./../../handler/fundingaccount/DeleteFundingAccountHandler'), require('./../../handler/fundingaccount/GetFundingAccountHandler'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../util/ResponseWrapper'),require('./../../handler/fundingaccount/ReplaceFundingAccountHandler'));
    }
}(this, function (logger, Address, Base, InvocationContext, RestConstants, ProductEnum, Attributes, InvocationModeEnum, AddFundingAccountHandler, UpdateFundingAccountHandler, DeleteFundingAccountHandler, GetFundingAccountHandler, OrbipayApiException, ValidationUtil, ResponseWrapper,ReplaceFundingAccountHandler) {
        'use strict';


        /**
         * The FundingAccount model module.
         * @module model/fundingaccount/FundingAccount
         * @version 1.0.0
         * @param id {String} The unique identifier assigned by Orbipay to the paymentsetup.
         */

        /**
         * Constructs a new <code>FundingAccount</code>.
         * @alias module:model/fundingaccount/FundingAccount
         * @class
         * @param ID_FUNDING_ACCOUNT {String} Id is a unique identifier assigned to the funding account in Orbipay. A new Id is generated for the funding account when the account number is updated.
         */
        var exports = function (ID_FUNDING_ACCOUNT) {
            var _this = this;
            Base.call(_this);
            _this['ID_FUNDING_ACCOUNT'] = ID_FUNDING_ACCOUNT;
            _this['id'] = ID_FUNDING_ACCOUNT;
        };

        /**
         * @private
         * Constructs a <code>FundingAccount</code> from a plain JavaScript object, optionally creating a new instance.
         * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
         * @param {Object} data The plain JavaScript object bearing properties of interest.
         * @param {module:model/fundingaccount/FundingAccount} obj Optional instance to populate.
         * @return {module:model/fundingaccount/FundingAccount} The populated <code>FundingAccount</code> instance.
         */
        exports.constructFromObject = function (data, obj) {
            if (data) {
                obj = obj || new exports();
                if (data.hasOwnProperty('account_holder_name')) {
                    obj['account_holder_name'] = data['account_holder_name'];
                }
                if (data.hasOwnProperty('nickname')) {
                    obj['nickname'] = data['nickname'];
                }
                if (data.hasOwnProperty('address')) {
                    obj['address'] = Address.constructFromObject(data['address']);
                }
                if (data.hasOwnProperty('account_number')) {
                    obj['account_number'] = data['account_number'];
                }
                if (data.hasOwnProperty('aba_routing_number')) {
                    obj['aba_routing_number'] = data['aba_routing_number'];
                }
                if (data.hasOwnProperty('issuer_name')) {
                    obj['issuer_name'] = data['issuer_name'];
                }
                if (data.hasOwnProperty('currency_code3d')) {
                    obj['currency_code3d'] = data['currency_code3d'];
                }
                if (data.hasOwnProperty('expiry_date')) {
                    obj['expiry_date'] = data['expiry_date'];
                }
                if (data.hasOwnProperty('account_holder_type')) {
                    obj['account_holder_type'] = data['account_holder_type'];
                }
                if (data.hasOwnProperty('account_subtype')) {
                    obj['account_subtype'] = data['account_subtype'];
                }
                if (data.hasOwnProperty('id')) {
                    obj['id'] = data['id'];
                }
                if (data.hasOwnProperty('url')) {
                    obj['url'] = data['url'];
                }
                if (data.hasOwnProperty('ach_eligible_flag')) {
                    obj['ach_eligible_flag'] = data['ach_eligible_flag'];
                }
                if (data.hasOwnProperty('atm_eligible_flag')) {
                    obj['atm_eligible_flag'] = data['atm_eligible_flag'];
                }
                if (data.hasOwnProperty('card_cvv_number')) {
                    obj['card_cvv_number'] = data['card_cvv_number'];
                }
                if (data.hasOwnProperty('account_type')) {
                    obj['account_type'] = data['account_type'];
                }
                if (data.hasOwnProperty('status')) {
                    obj['status'] = data['status'];
                }
                if (data.hasOwnProperty('custom_fields')) {
                    obj['custom_fields'] = data['custom_fields'];
                }
                if (data.hasOwnProperty('comments')) {
                    obj['comments'] = data['comments'];
                }
                if (data.hasOwnProperty('deleted')) {
                    obj['deleted'] = data['deleted'];
                }
            }
            return obj;
        };

        exports.prototype = Object.create(Base.prototype);

        exports.prototype.constructor = exports;

        /**
         * The name as specified on the account.
         * @member {String} account_holder_name
         */
        exports.prototype['account_holder_name'] = undefined;
        /**
         * The nickname by which a customer might want to identify the account.
         * @member {String} nickname
         */
        exports.prototype['nickname'] = undefined;
        /**
         * @member {module:model/base/Address} address
         */
        exports.prototype['address'] = undefined;
        /**
         * The number or reference that a customer uses to identify the funding account.
         * @member {String} account_number
         */
        exports.prototype['account_number'] = undefined;
        /**
         * The ABA/Routing number for the bank account.
         * @member {String} aba_routing_number
         */
        exports.prototype['aba_routing_number'] = undefined;
        /**
         * The name of the authority that has issued the account. This is relevant only in the case of bank accounts.
         * @member {String} issuer_name
         */
        exports.prototype['issuer_name'] = undefined;
        /**
         * The three-letter currency code in the ISO4217 format, in uppercase, for the card account. It must be a currency supported by Orbipay. This defaults to USD.
         * @member {String} currency_code3d
         */
        exports.prototype['currency_code3d'] = undefined;
        /**
         * The expiry date for the card. This is to be in the format MM/yy. Only applicable for cards accounts.
         * @member {String} expiry_date
         */
        exports.prototype['expiry_date'] = undefined;
        /**
         * The type of ownership for the funding account. This is applicable only in the case of bank accounts and it is defaults to personal.
         * @member {module:model/fundingaccount/FundingAccount.AccountHolderTypeEnum} account_holder_type
         */
        exports.prototype['account_holder_type'] = undefined;
        /**
         * The sub type of the funding account. This is derived by Orbipay in the case of card funding accounts. It needs to be passed when adding or editing a DDA/bank account. For e.g., a savings account, a checking account.
         * @member {module:model/fundingaccount/FundingAccount.AccountSubtypeEnum} account_subtype
         */
        exports.prototype['account_subtype'] = undefined;
        /**
         * Id is a unique identifier assigned to the funding account in Orbipay. A new Id is generated for the funding account when the account number is updated.
         * @member {String} id
         */
        exports.prototype['id'] = undefined;
        /**
         * This URL fetches the Account details.
         * @member {String} url
         */
        exports.prototype['url'] = undefined;
        /**
         * Indicates if the funding account can be debited vide ACH, in case the funding account is a Demand Deposit Account (DDA).
         * @member {module:model/fundingaccount/FundingAccount.AchEligibleFlagEnum} ach_eligible_flag
         */
        exports.prototype['ach_eligible_flag'] = undefined;
        /**
         * Indicates if the funding account can be debited vide the ATM Rail network, in case the funding account is a Debit Card.
         * @member {module:model/fundingaccount/FundingAccount.AtmEligibleFlagEnum} atm_eligible_flag
         */
        exports.prototype['atm_eligible_flag'] = undefined;
        /**
         * This is never returned by Orbipay. This needs to be passed to Orbipay when adding or editing a card funding account.
         * @member {String} card_cvv_number
         */
        exports.prototype['card_cvv_number'] = undefined;
        /**
         * The type of the funding account.
         * @member {module:model/fundingaccount/FundingAccount.AccountTypeEnum} account_type
         */
        exports.prototype['account_type'] = undefined;
        /**
         * The status of the funding account in Orbipay.
         * @member {module:model/fundingaccount/FundingAccount.StatusEnum} status
         */
        exports.prototype['status'] = undefined;
        /**
         * The additional information or meta-information that Orbipay can accept, maintain and transmit back to the client. The custom fields need to be configured with Orbipay before they can be accepted. Orbipay would reject custom fields that are not pre-configured. Please contact support@billerpayments.com for more information on configuring and using custom fields.
         * @member {Object} custom_fields
         */
        exports.prototype['custom_fields'] = undefined;
        /**
         * @member {String} deleted
         */
        exports.prototype['deleted'] = undefined;

        /**
         * @param {String} ID_CUSTOMER
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.forCustomer = function (ID_CUSTOMER) {
            var _this = this;
            _this['ID_CUSTOMER'] = ID_CUSTOMER;
            return _this;
        };

        /**
         * @param {module:model/base/Address} address
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.withAddress = function (address) {
            var _this = this;
            _this['address'] = Address.constructFromObject(address);
            return _this;
        };

        /**
         * @param {String} comments.
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.withMemo = function (comments) {
            var _this = this;
            _this['comments'] = comments;
            return _this;
        };

        /**
         * @param {String} requestor
         * @param {String} requestor_type
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.deletedBy = function (requestor, requestor_type) {
            var _this = this;
            _this.requestedBy(requestor, requestor_type);
            return _this;
        };

        /**
         * @param {String} requestor
         * @param {String} requestor_type
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.createdBy = function (requestor, requestor_type) {
            var _this = this;
            _this.requestedBy(requestor, requestor_type);
            return _this;
        };

        /**
         * @param {String} requestor
         * @param {String} requestor_type
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.retrievedBy = function (requestor, requestor_type) {
            var _this = this;
            _this.requestedBy(requestor, requestor_type);
            return _this;
        };

        /**
         * @param {String} requestor
         * @param {String} requestor_type
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.updatedBy = function (requestor, requestor_type) {
            var _this = this;
            _this.requestedBy(requestor, requestor_type);
            return _this;
        };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/fundingaccount/FundingAccount
     */
    exports.prototype.replacedBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };

        /**
         * @param {String} cardCvvNumber
         * @param {String} accountHolderName
         * @param {String} nickname
         * @param {String} accountNumber
         * @param {String} status
         * @param {String} abaRoutingNumber
         * @param {String} accountSubtype
         * @param {String} expiryDate
         * @param {String} accountHolderType
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.withDetails = function (cardCvvNumber, accountHolderName, nickname, accountNumber, status, abaRoutingNumber,
                                                  accountSubtype, expiryDate, accountHolderType) {
            var _this = this;
            _this['card_cvv_number'] = cardCvvNumber;
            _this['account_holder_name'] = accountHolderName;
            _this['nickname'] = nickname;
            _this['account_number'] = accountNumber;
            _this['status'] = status;
            _this['aba_routing_number'] = abaRoutingNumber;
            _this['account_subtype'] = accountSubtype;
            _this['expiry_date'] = expiryDate;
            _this['account_holder_type'] = accountHolderType;
            return this;
        };

        /**
         * @param {Object} custom_fields
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.withCustomFields = function (custom_fields) {
            var _this = this;
            _this['custom_fields'] = custom_fields;
            return _this;
        };

        /**
         * @param {String} status. Use {module:enum/FundingAccountStatusEnum} to get values.
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.withStatus = function (status) {
            var _this = this;
            _this['status'] = status;
            return _this;
        };

        /**
         * @param {String} channel
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.videChannel = function (channel) {
            var _this = this;
            Base.prototype.videChannel.call(_this, channel);
            return _this;
        };

        /**
         * @param {String} client_key
         * @return module:model/fundingaccount/FundingAccount
         */
        exports.prototype.forClient = function (client_key) {
            var _this = this;
            Base.prototype.forClient.call(_this, client_key);
            return _this;
        };

        /**
         * Callback function to receive the result of the operation.
         * @callback module:model/fundingaccount/FundingAccount~callApiCallback
         * @param {module:exception/OrbipayApiException} exception Error Response, if any.
         * @param {module:model/fundingaccount/FundingAccount} data The data returned by the service call.
         */


        /**
         * @param {module:model/base/InvocationContext} invocation_context
         * @param {module:model/fundingaccount/FundingAccount~callApiCallback} callback
         * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
         * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
         * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
         */
        exports.prototype.create = function (invocation_context, callback, invocation_mode, api_end_point) {

            var handler;
            var _this = this;
            var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
                try {
                    var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                    if (callback) {
                        callback(response['exception'], response['data'])
                    }
                } catch (e) {
                    exception = OrbipayApiException.getDefaultException(e, _this);
                    if (callback) {
                        callback(exception)
                    }
                }
            };

            _this.withContext(invocation_context, invocation_mode, api_end_point);
            handler = new AddFundingAccountHandler();

            if (callback && ValidationUtil.isFunction(callback)) {
                return handler.handleRequest(_this, internalCallback);
            } else {
                return handler.handleRequest(_this);
            }
        };

        /**
         * @param {module:model/base/InvocationContext} invocation_context
         * @param {module:model/fundingaccount/FundingAccount~callApiCallback} callback
         * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
         * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
         * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
         */
        exports.prototype.get = function (invocation_context, callback, invocation_mode, api_end_point) {

            var handler;
            var _this = this;
            var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
                try {
                    var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                    if (callback) {
                        callback(response['exception'], response['data'])
                    }
                } catch (e) {
                    exception = OrbipayApiException.getDefaultException(e, _this);
                    if (callback) {
                        callback(exception)
                    }
                }
            };

            _this.withContext(invocation_context, invocation_mode, api_end_point);
            handler = new GetFundingAccountHandler();

            if (callback && ValidationUtil.isFunction(callback)) {
                return handler.handleRequest(_this, internalCallback);
            } else {
                return handler.handleRequest(_this);
            }
        };

        /**
         * @param {module:model/base/InvocationContext} invocation_context
         * @param {module:model/fundingaccount/FundingAccount~callApiCallback} callback
         * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
         * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
         * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
         */
        exports.prototype.update = function (invocation_context, callback, invocation_mode, api_end_point) {

            var handler;
            var _this = this;
            var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
                try {
                    var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                    if (callback) {
                        callback(response['exception'], response['data'])
                    }
                } catch (e) {
                    exception = OrbipayApiException.getDefaultException(e, _this);
                    if (callback) {
                        callback(exception)
                    }
                }
            };

            _this.withContext(invocation_context, invocation_mode, api_end_point);
            handler = new UpdateFundingAccountHandler();

            if (callback && ValidationUtil.isFunction(callback)) {
                return handler.handleRequest(_this, internalCallback);
            } else {
                return handler.handleRequest(_this);
            }
        };

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/fundingaccount/FundingAccount~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.replace = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;
        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new ReplaceFundingAccountHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

        /**
         * @param {module:model/base/InvocationContext} invocation_context
         * @param {module:model/fundingaccount/FundingAccount~callApiCallback} callback
         * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
         * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
         * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
         */
        exports.prototype.delete = function (invocation_context, callback, invocation_mode, api_end_point) {

            var handler;
            var _this = this;
            var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
                try {
                    var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                    if (callback) {
                        callback(response['exception'], response['data'])
                    }
                } catch (e) {
                    exception = OrbipayApiException.getDefaultException(e, _this);
                    if (callback) {
                        callback(exception)
                    }
                }
            };

            _this.withContext(invocation_context, invocation_mode, api_end_point);
            handler = new DeleteFundingAccountHandler();

            if (callback && ValidationUtil.isFunction(callback)) {
                return handler.handleRequest(_this, internalCallback);
            } else {
                return handler.handleRequest(_this);
            }
        };
        return exports;
    }
));


