/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/paymentsetup/PaymentSetup'), require('../../swagger/ApiClient'), require('./../base/Base'), require('./../base/InvocationContext'), require('../../constant/RestConstants'), require('../../handler/paymentsetup/CreatePaymentSetupHandler'), require('../../model/paymentsetup/PaymentSchedule'), require('../../model/payment/Fee'), require('../../model/fundingaccount/FundingAccount'), require('../../model/customeraccount/CustomerAccount'), require('../../model/customer/Customer'), require('../../model/base/Attributes'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../enum/ProductEnum'), require('./../../enum/InvocationModeEnum'), require('./../../util/ValidationUtil'), require('../../handler/paymentsetup/GetPaymentSetupHandler'), require('../../handler/paymentsetup/DeletePaymentSetupHandler'), require('./../../util/ResponseWrapper'),require('../../model/payment/PaymentNetworkResponse'), require('../../handler/paymentsetup/UpdatePaymentSetupHandler'));
    }
}(this, function (logger, ApiClient, Base, InvocationContext, RestConstants, CreatePaymentSetupHandler, PaymentSchedule, Fee, FundingAccount, CustomerAccount, Customer, Attributes, CommonUtil, OrbipayApiException, ProductEnum, InvocationModeEnum, ValidationUtil, GetPaymentSetupHandler, DeletePaymentSetupHandler, ResponseWrapper, PaymentNetworkResponse, UpdatePaymentSetupHandler) {
    'use strict';


    /**
     * The PaymentSetup model module.
     * @module model/paymentsetup/PaymentSetup
     * @version 1.0.0s
     */

    /**
     * Constructs a new <code>PaymentSetup</code>.
     * @alias module:model/paymentsetup/PaymentSetup
     * @class
     * @param ID_PAYMENT_SETUP {String} The unique identifier assigned by Orbipay to the paymentsetup.
     */
    var exports = function (ID_PAYMENT_SETUP) {
        var _this = this;
        Base.call(this);
        _this['ID_PAYMENT_SETUP'] = ID_PAYMENT_SETUP;
        _this['id'] = ID_PAYMENT_SETUP;
    };

    /**
     * @private
     * Constructs a <code>PaymentSetup</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/paymentsetup/PaymentSetup} obj Optional instance to populate.
     * @return {module:model/paymentsetup/PaymentSetup} The populated <code>PaymentSetup</code>  instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('comments')) {
                obj['comments'] = data['comments'];
            }
            if (data.hasOwnProperty('id')) {
                obj['id'] = data['id'];
            }
            if (data.hasOwnProperty('url')) {
                obj['url'] = data['url'];
            }
            if (data.hasOwnProperty('fee')) {
                obj['fee'] = Fee.constructFromObject(data['fee']);
            }
            if (data.hasOwnProperty('status')) {
                obj['status'] = data['status'];
            }
            if (data.hasOwnProperty('payment_method')) {
                obj['payment_method'] = data['payment_method'];
            }
            if (data.hasOwnProperty('payment_setup_schedule_type')) {
                obj['payment_setup_schedule_type'] = data['payment_setup_schedule_type'];
            }
            if (data.hasOwnProperty('confirmation_number')) {
                obj['confirmation_number'] = data['confirmation_number'];
            }
            if (data.hasOwnProperty('currency_code3d')) {
                obj['currency_code3d'] = data['currency_code3d'];
            }
            if (data.hasOwnProperty('amount')) {
                obj['amount'] = data['amount'];
            }
            if (data.hasOwnProperty('card_cvv_number')) {
                obj['card_cvv_number'] = data['card_cvv_number'];
            }
            if (data.hasOwnProperty('next_payment_date')) {
                obj['next_payment_date'] = data['next_payment_date'];
            }
            if (data.hasOwnProperty('payment_setup_reference')) {
                obj['payment_setup_reference'] = data['payment_setup_reference'];
            }
            if (data.hasOwnProperty('payment_schedule')) {
                obj['payment_schedule'] = PaymentSchedule.constructFromObject(data['payment_schedule']);
            }
            if (data.hasOwnProperty('payment_setup_entry_date')) {
                obj['payment_setup_entry_date'] = data['payment_setup_entry_date'];
            }
            if (data.hasOwnProperty('customer')) {
                obj['customer'] = Customer.constructFromObject(data['customer']);
            }
            if (data.hasOwnProperty('funding_account')) {
                obj['funding_account'] = FundingAccount.constructFromObject(data['funding_account']);
            }
            if (data.hasOwnProperty('customer_account')) {
                obj['customer_account'] = CustomerAccount.constructFromObject(data['customer_account']);
            }
            if (data.hasOwnProperty('custom_fields')) {
                obj['custom_fields'] = data['custom_fields'];
            }
            if (data.hasOwnProperty('deleted')) {
                obj['deleted'] = data['deleted'];
            }
            if (data.hasOwnProperty('payment_network_response')) {
                obj['payment_network_response'] = PaymentNetworkResponse.constructFromObject(data['payment_network_response']);
            }
        }
        return obj;
    };

    exports.prototype = Object.create(Base.prototype);

    exports.prototype.constructor = exports;

    /**
     * @member {String} comments
     */
    exports.prototype['comments'] = undefined;
    /**
     * The unique identifier assigned by Orbipay to the paymentsetup.
     * @member {String} id
     */
    exports.prototype['id'] = undefined;
    /**
     * This URL fetches the details of paymentsetup.
     * @member {String} url
     */
    exports.prototype['url'] = undefined;
    /**
     * @member {module:model/payment/Fee} fee
     */
    exports.prototype['fee'] = undefined;
    /**
     * The status of the paymentsetup.
     * @member {module:model/paymentsetup/PaymentSetup.StatusEnum} status
     */
    exports.prototype['status'] = undefined;
    /**
     * The fulfillment method for the paymentsetup.
     * @member {module:model/paymentsetup/PaymentSetup.PaymentMethodEnum} payment_method
     */
    exports.prototype['payment_method'] = undefined;
    /**
     * The schedule type for the payment setup.
     * @member {module:model/paymentsetup/PaymentSetup.PaymentSetupScheduleTypeEnum} payment_setup_schedule_type
     */
    exports.prototype['payment_setup_schedule_type'] = undefined;
    /**
     * The confirmation number or reference provided to the customer for the paymentsetup.
     * @member {String} confirmation_number
     */
    exports.prototype['confirmation_number'] = undefined;
    /**
     * The three-letter currency code in the ISO4217 format, in uppercase, for the card account. It must be a currency supported by Orbipay. This defaults to USD.
     * @member {String} currency_code3d
     */
    exports.prototype['currency_code3d'] = undefined;
    /**
     * The amount being paid. In case the payment_amount_type is 'discounted_amount', 'penalty_amount' or 'other', the amount needs to be specified. In other cases, this is derived from the payment_amount_type.
     * @member {String} amount
     */
    exports.prototype['amount'] = undefined;
    /**
     * This is never returned by Orbipay. This needs to be passed to Orbipay when making or editing a paymentsetup with a card funding account.
     * @member {String} card_cvv_number
     */
    exports.prototype['card_cvv_number'] = undefined;
    /**
     * The date on which the payment is scheduled to be made. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.
     * @member {String} next_payment_date
     */
    exports.prototype['next_payment_date'] = undefined;
    /**
     * The unique identifier in the client system for the paymentSetup.
     * @member {String} payment_setup_reference
     */
    exports.prototype['payment_setup_reference'] = undefined;
    /**
     * @member {module:model/paymentsetup/PaymentSchedule} payment_schedule
     */
    exports.prototype['payment_schedule'] = undefined;
    /**
     * The date on which the paymentsetup is captured in Orbipay. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.
     * @member {String} payment_setup_entry_date
     */
    exports.prototype['payment_setup_entry_date'] = undefined;
    /**
     * @member {module:model/customer/Customer} customer
     */
    exports.prototype['customer'] = undefined;
    /**
     * @member {module:model/fundingaccount/FundingAccount} funding_account
     */
    exports.prototype['funding_account'] = undefined;
    /**
     * @member {module:model/customeraccount/CustomerAccount} customer_account
     */
    exports.prototype['customer_account'] = undefined;
    /**
     * The additional information or meta-information that Orbipay can accept, maintain and transmit back to the client. The custom fields need to be configured with Orbipay before they can be accepted. Orbipay would reject custom fields that are not pre-configured. Please contact support@billerpayments.com for more information on configuring and using custom fields.
     * @member {Object} custom_fields
     */
    exports.prototype['custom_fields'] = undefined;
    /**
     * @member {String} deleted
     */
    exports.prototype['deleted'] = undefined;
    /**
     * @member {module:model/payment/PaymentNetworkResponse} payment_network_response
     */
    exports.prototype['payment_network_response'] = undefined;

    /**
     * @param {String} payment_setup_reference.
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.withReference = function (payment_setup_reference) {
        this['payment_setup_reference'] = payment_setup_reference;
        return this;
    };

    /**
     * @param {Object} custom_fields.
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.withCustomFields = function (custom_fields) {
        this['custom_fields'] = custom_fields;
        return this;
    };

    /**
     * @param {module:model/payment/Fee} fee.
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.withFee = function (fee) {
        this['fee'] = Fee.constructFromObject(fee);
        return this;
    };

    /**
     * @param {String} comments.
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.withMemo = function (comments) {
        this['comments'] = comments;
        return this;
    };

    /**
     * @param {String} amount
     * @param {String} card_cvv_number
     * @param {String} payment_setup_schedule_type
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.withDetails = function (amount, card_cvv_number, payment_setup_schedule_type) {
        this['amount'] = amount;
        this['card_cvv_number'] = card_cvv_number;
        this['payment_setup_schedule_type'] = payment_setup_schedule_type;
        return this;
    };

    /**
     * @param {String} ID_CUSTOMER
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.forCustomer = function (ID_CUSTOMER) {
        var idCustomer = ID_CUSTOMER;
        this['ID_CUSTOMER'] = idCustomer;
        this['customer'] = new Customer(idCustomer);
        return this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.createdBy = function (requestor, requestor_type) {
        this.requestedBy(requestor, requestor_type);
        return this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.retrievedBy = function (requestor, requestor_type) {
        this.requestedBy(requestor, requestor_type);
        return this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.deletedBy = function (requestor, requestor_type) {
        this.requestedBy(requestor, requestor_type);
        return this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.updatedBy = function (requestor, requestor_type) {
        this.requestedBy(requestor, requestor_type);
        return this;
    };

    /**
     * @param {String} channel
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.videChannel = function (channel) {
        var _this = this;
        Base.prototype.videChannel.call(_this, channel);
        return _this;
    };

    /**
     * @param {String} client_key
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.forClient = function (client_key) {
        var _this = this;
        Base.prototype.forClient.call(_this, client_key);
        return _this;
    };

    /**
     * @param {module:model/paymentsetup/PaymentSchedule} payment_schedule
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.withPaymentSchedule = function (payment_schedule) {
        var _this = this;
        _this['payment_schedule'] = PaymentSchedule.constructFromObject(payment_schedule);
        return _this;
    };

    /**
     * @param {module:model/customeraccount/CustomerAccount} customer_account
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.toAccount = function (customer_account) {
        var _this = this;
        _this['customer_account'] = CustomerAccount.constructFromObject(customer_account);
        return _this;
    };

    /**
     * @param {module:model/fundingaccount/FundingAccount} funding_account
     * @return module:model/paymentsetup/PaymentSetup
     */
    exports.prototype.fromAccount = function (funding_account) {
        var _this = this;
        _this['funding_account'] = FundingAccount.constructFromObject(funding_account);
        return _this;
    };

    /**
     * Callback function to receive the result of the operation.
     * @callback module:model/paymentsetup/PaymentSetup~callApiCallback
     * @param {module:exception/OrbipayApiException} exception Error Response, if any.
     * @param {module:model/paymentsetup/PaymentSetup} data The data returned by the service call.
     */


    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/paymentsetup/PaymentSetup~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.create = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new CreatePaymentSetupHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/paymentsetup/PaymentSetup~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.get = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;
        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new GetPaymentSetupHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/paymentsetup/PaymentSetup~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.delete = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;
        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new DeletePaymentSetupHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/paymentsetup/PaymentSetup~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.update = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new UpdatePaymentSetupHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    return exports;
}));


