/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/paymentsetup/PaymentSetups'), require('../../swagger/ApiClient'), require('./PaymentSetup'), require('./../base/Base'), require('./../base/InvocationContext'), require('./../../enum/InvocationModeEnum'), require('./../../constant/RestConstants'), require('./../../model/base/Attributes'), require('./../../enum/ProductEnum'), require('./../../util/CommonUtil'), require('./../../util/ValidationUtil'), require('./../../handler/paymentsetup/RetrievePaymentSetupsHandler'), require('../../exception/OrbipayApiException'), require('./../../util/ResponseWrapper'));
    }
}(this, function (logger, ApiClient, PaymentSetup, Base, InvocationContext, InvocationModeEnum, RestConstants, Attributes, ProductEnum, CommonUtil, ValidationUtil, RetrievePaymentSetupsHandler, OrbipayApiException, ResponseWrapper) {
    'use strict';


    /**
     * The PaymentSetups model module.
     * @module model/paymentsetup/PaymentSetups
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>PaymentSetups</code>.
     * @alias module:model/paymentsetup/PaymentSetups
     * @class
     */
    var exports = function () {
        var _this = this;
        Base.call(_this);
    };

    /**
     * @private
     * Constructs a <code>PaymentSetups</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/paymentsetup/PaymentSetups} obj Optional instance to populate.
     * @return {module:model/paymentsetup/PaymentSetups} The populated <code>PaymentSetups</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('list')) {
                obj['list'] = ApiClient.convertToType(data['list'], [PaymentSetup]);
            }
            if (data.hasOwnProperty('total_results_count')) {
                obj['total_results_count'] = data['total_results_count'];
            }
            if (data.hasOwnProperty('has_more_results')) {
                obj['has_more_results'] = data['has_more_results'];
            }
            if (data.hasOwnProperty('url')) {
                obj['url'] = data['url'];
            }
            if (data.hasOwnProperty('from_index')) {
                obj['from_index'] = data['from_index'];
            }
            if (data.hasOwnProperty('to_index')) {
                obj['to_index'] = data['to_index'];
            }
        }
        return obj;
    };

    exports.prototype = Object.create(Base.prototype);

    exports.prototype.constructor = exports;

    /**
     * @member {Array.<module:model/paymentsetup/PaymentSetup>} list
     */
    exports.prototype['list'] = undefined;
    /**
     * The total number of payment setup available matching the search criteria.
     * @member {String} total_results_count
     */
    exports.prototype['total_results_count'] = undefined;
    /**
     * true if there are more results or pages
     * @member {module:model/paymentsetup/PaymentSetups.HasMoreResultsEnum} has_more_results
     */
    exports.prototype['has_more_results'] = undefined;
    /**
     * This URL fetches the next or previous set of payment setups, based on the presence of after_object or before_object parameter.
     * @member {String} url
     */
    exports.prototype['url'] = undefined;
    /**
     * The id of the object before which the previous set of objects are to be retrieved.
     * @member {String} from_index
     */
    exports.prototype['from_index'] = undefined;
    /**
     * The id of the object after which the next set of objects are to be retrieved.
     * @member {String} to_index
     */
    exports.prototype['to_index'] = undefined;

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/paymentsetup/PaymentSetups
     */
    exports.prototype.retrievedBy = function (requestor, requestor_type) {
        this.requestedBy(requestor, requestor_type);
        return this;
    };

    /**
     * @param {String} channel
     * @return module:model/paymentsetup/PaymentSetups
     */
    exports.prototype.videChannel = function (channel) {
        var _this = this;
        Base.prototype.videChannel.call(_this, channel);
        return _this;
    };

    /**
     * @param {String} client_key
     * @return module:model/paymentsetup/PaymentSetups
     */
    exports.prototype.forClient = function (client_key) {
        var _this = this;
        Base.prototype.forClient.call(_this, client_key);
        return _this;
    };

    /**
     * @private
     * @param {module:model/base/MultiMap} filterParameters.
     * @return module:model/paymentsetup/PaymentSetups
     */
    exports.prototype.findBy = function (filterParameters) {
        var _this = this;
        if (filterParameters) {
            _this['page_size'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'page_size', false);
            _this['id_customer'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'id_customer', false);
            _this['id_funding_account'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'id_funding_account', false);
            _this['id_customer_account'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'id_customer_account', false);
            _this['confirmation_number'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'confirmation_number', false);
            _this['status'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'status', true);
            _this['payment_setup_schedule_type'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'payment_setup_schedule_type', true);
            _this['from_date'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'from_date', false);
            _this['to_date'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'to_date', false);
            _this['query_id'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'query_id', false);
            _this['from_index'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'from_index', false);
            _this['to_index'] = CommonUtil.getValuesFromMultiMap(filterParameters, 'to_index', false);
        }
        return this;
    };

    /**
     * Callback function to receive the result of the operation.
     * @callback module:model/paymentsetup/PaymentSetups~callApiCallback
     * @param {module:exception/OrbipayApiException} exception Error Response, if any.
     * @param {module:model/paymentsetup/PaymentSetups} data The data returned by the service call.
     */

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/paymentsetup/PaymentSetups~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.get = function (invocation_context, callback, invocation_mode, api_end_point) {
        var handler;
        var _this = this;

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new RetrievePaymentSetupsHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * onDone function to notify end of pagination.
     * @callback module:model/paymentsetup/PaymentSetups~onDone
     * @param {module:exception/OrbipayApiException} exception Error Response, if any.
     */

    /**
     * onItem function to receive each item.
     * @callback module:model/paymentsetup/PaymentSetups~onItem
     * @param {module:model/paymentsetup/PaymentSetup} PaymentSetup object.
     * @param {module:model/paymentsetup/PaymentSetups} data The data returned by the service call.
     */

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/paymentsetup/PaymentSetups~onItem} onItem
     * @param {module:model/paymentsetup/PaymentSetups~onDone} onDone
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.getAutoPagingIterator = function (invocation_context, onItem, onDone, invocation_mode, api_end_point) {
        var handler;
        var _this = this;
        var continuePagination = true;

        /**
         * next function to break out of the loop. Pass `false` to break out of the loop.
         * @callback module:model/paymentsetup/PaymentSetups~getAutoPagingIterator~next
         * @param {Boolean} isContinue.
         */
        var next = function (isContinue) {
            if (isContinue === false) {
                continuePagination = false;
            }
        };

        function clearPreviousFilterParams() {
            _this['id_funding_account'] = undefined;
            _this['id_customer_account'] = undefined;
            _this['confirmation_number'] = undefined;
            _this['status'] = undefined;
            _this['payment_setup_schedule_type'] = undefined;
            _this['from_date'] = undefined;
            _this['to_date'] = undefined;
            _this['to_index'] = undefined;
        }

        function setQueryInfo(data) {
            var url = data['url'];
            var fromIndex = parseInt(data['to_index']) + 1;

            _this['query_id'] = url ? url.substring(url.indexOf("=") + 1) : undefined;
            _this['from_index'] = '' + fromIndex;
        }

        function getNextPage(data) {
            setQueryInfo(data);
            //clear the previous filter parameters
            clearPreviousFilterParams();
            handler.handleRequest(_this, internalCallback);
        }

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                exception = response['exception'];
                if (exception) {
                    onDone(exception);
                } else {
                    if (continuePagination === true && response['data'] && response['data']['list'] && response['data']['list'].length > 0) {
                        var data = response['data'];
                        var list = data['list'];
                        var index = 0;
                        for (; index < list.length; index++) {
                            if (continuePagination === false) {
                                break;
                            }
                            onItem(list[index], next);
                        }
                        if (continuePagination === true && index === list.length && data['has_more_results'] === 'true') {
                            getNextPage(data);
                        } else {
                            onDone();
                        }
                    } else {
                        onDone();
                    }
                }
            } catch (e) {
                logger.error(e);
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (onDone) {
                    onDone(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new RetrievePaymentSetupsHandler();

        if (onItem && ValidationUtil.isFunction(onItem) && onDone && ValidationUtil.isFunction(onDone)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    return exports;
}));


