/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/refund/Refund'), require('./../payment/Payment'), require('./FeeRefund'), require('./../base/Base'), require('./../base/InvocationContext'), require('./../../enum/ProductEnum'), require('./../base/Attributes'), require('./../../constant/RestConstants'), require('./../../enum/InvocationModeEnum'), require('./../../handler/refund/CreateRefundHandler'), require('./../../util/ValidationUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ResponseWrapper'), require('../../model/payment/PaymentNetworkResponse'), require('./../../handler/refund/GetRefundHandler'));
    }
}(this, function (logger, Payment, FeeRefund, Base, InvocationContext, ProductEnum, Attributes, RestConstants, InvocationModeEnum, CreateRefundHandler, ValidationUtil, OrbipayApiException, ResponseWrapper, PaymentNetworkResponse, GetRefundHandler) {
    'use strict';


    /**
     * The Refund model module.
     * @module model/refund/Refund
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>Refund</code>.
     * @alias module:model/refund/Refund
     * @param ID_REFUND {String} The unique identifier assigned by Orbipay to the refund.
     * @class
     */
    var exports = function (ID_REFUND) {
        var _this = this;
        Base.call(_this);
        _this['ID_REFUND'] = ID_REFUND;
        _this['id'] = ID_REFUND;
    };

    /**
     * @private
     * Constructs a <code>Refund</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/refund/Refund} obj Optional instance to populate.
     * @return {module:model/refund/Refund} The populated <code>Refund</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('source_payment')) {
                obj['source_payment'] = Payment.constructFromObject(data['source_payment']);
            }
            if (data.hasOwnProperty('id')) {
                obj['id'] = data['id'];
            }
            if (data.hasOwnProperty('url')) {
                obj['url'] = data['url'];
            }
            if (data.hasOwnProperty('expected_refund_settlement_date')) {
                obj['expected_refund_settlement_date'] = data['expected_refund_settlement_date'];
            }
            if (data.hasOwnProperty('status')) {
                obj['status'] = data['status'];
            }
            if (data.hasOwnProperty('confirmation_number')) {
                obj['confirmation_number'] = data['confirmation_number'];
            }
            if (data.hasOwnProperty('refund_reference')) {
                obj['refund_reference'] = data['refund_reference'];
            }
            if (data.hasOwnProperty('refund_type')) {
                obj['refund_type'] = data['refund_type'];
            }
            if (data.hasOwnProperty('refund_amount')) {
                obj['refund_amount'] = data['refund_amount'];
            }
            if (data.hasOwnProperty('custom_fields')) {
                obj['custom_fields'] = data['custom_fields'];
            }
            if (data.hasOwnProperty('fee_refund')) {
                obj['fee_refund'] = FeeRefund.constructFromObject(data['fee_refund']);
            }
            if (data.hasOwnProperty('payment_network_response')) {
                obj['payment_network_response'] = PaymentNetworkResponse.constructFromObject(data['payment_network_response']);
            }
            if (data.hasOwnProperty('refund_date')) {
                obj['refund_date'] = data['refund_date'];
            }
            if (data.hasOwnProperty('refund_entry_date')) {
                obj['refund_entry_date'] = data['refund_entry_date'];
            }
            /**
             * @member {module:model/payment/PaymentNetworkResponse} payment_network_response
             */
            exports.prototype['payment_network_response'] = undefined;
        }
        return obj;
    };

    exports.prototype = Object.create(Base.prototype);

    exports.prototype.constructor = exports;

    /**
     * @member {module:model/payment/Payment} source_payment
     */
    exports.prototype['source_payment'] = undefined;
    /**
     * The unique identifier assigned by Orbipay to the refund.
     * @member {String} id
     */
    exports.prototype['id'] = undefined;
    /**
     * This URL fetches the details of refund.
     * @member {String} url
     */
    exports.prototype['url'] = undefined;
    /**
     * The date by which the credits would be posted to the original funding account. ISO8601 full-date format, YYYY-MM-DD
     * @member {String} expected_refund_settlement_date
     */
    exports.prototype['expected_refund_settlement_date'] = undefined;

    exports.prototype['refund_entry_date'] = undefined;

    exports.prototype['refund_date'] = undefined;
    /**
     * The status of the refund.
     * @member {module:model/refund/Refund.StatusEnum} status
     */
    exports.prototype['status'] = undefined;
    /**
     * The confirmation number or reference provided to the customer for the refund.
     * @member {String} confirmation_number
     */
    exports.prototype['confirmation_number'] = undefined;
    /**
     * The unique identifier in the client system for the refund.
     * @member {String} refund_reference
     */
    exports.prototype['refund_reference'] = undefined;
    /**
     * The portion of the payment amount that is to be or has been refunded. The original payment can be refunded fully including the amount and the fee components, or partially, for just the amount or the fee component. Refund can also be initiated for an arbitrary amount limited by the original payment amount.
     * @member {module:model/refund/Refund.RefundTypeEnum} refund_type
     */
    exports.prototype['refund_type'] = undefined;
    /**
     * The refund amount upto 2 decimal places.
     * @member {String} refund_amount
     */
    exports.prototype['refund_amount'] = undefined;
    /**
     * The additional information or meta-information that Orbipay can accept, maintain and transmit back to the client. The custom fields need to be configured with Orbipay before they can be accepted. Orbipay would reject custom fields that are not pre-configured. Please contact support@billerpayments.com for more information on configuring and using custom fields.
     * @member {Object} custom_fields
     */
    exports.prototype['custom_fields'] = undefined;
    /**
     * @member {module:model/refund/FeeRefund} fee_refund
     */
    exports.prototype['fee_refund'] = undefined;

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/refund/Refund
     */
    exports.prototype.createdBy = function (requestor, requestor_type) {
        this.requestedBy(requestor, requestor_type);
        return this;
    };

    /**
     * @param {String} ID_CUSTOMER
     * @return module:model/refund/Refund
     */
    exports.prototype.forCustomer = function (ID_CUSTOMER) {
        var _this = this;
        _this['ID_CUSTOMER'] = ID_CUSTOMER;
        return _this;
    };

    /**
     * @param {Object} custom_fields.
     * @return module:model/refund/Refund
     */
    exports.prototype.withCustomFields = function (custom_fields) {
        this['custom_fields'] = custom_fields;
        return this;
    };

    /**
     * @param {String} ID_PAYMENT
     * @return module:model/refund/Refund
     */
    exports.prototype.forPayment = function (ID_PAYMENT) {
        var _this = this;
        _this['ID_PAYMENT'] = ID_PAYMENT;
        return _this;
    };


    /**
     * @param {String} channel
     * @return module:model/refund/Refund
     */
    exports.prototype.videChannel = function (channel) {
        var _this = this;
        Base.prototype.videChannel.call(_this, channel);
        return _this;
    };

    /**
     * @param {String} client_key
     * @return module:model/refund/Refund
     */
    exports.prototype.forClient = function (client_key) {
        var _this = this;
        Base.prototype.forClient.call(_this, client_key);
        return _this;
    };


    /**
     * @param refund_amount
     * @return module:model/refund/Refund
     */
    exports.prototype.withDetails = function (refund_amount) {
        var _this = this;
        _this['refund_amount'] = refund_amount;
        return _this;
    };

    exports.prototype.withFeeRefund = function (fee_refund) {
        var _this = this;
        _this['fee_refund'] = FeeRefund.constructFromObject(fee_refund);
        return _this;
    };

    /**
     * @param {String} refund_reference
     * @return module:model/refund/Refund
     */
    exports.prototype.withReference = function (refund_reference) {
        var _this = this;
        _this['refund_reference'] = refund_reference;
        return _this;
    };

    /**
     * @return module:model/refund/Refund
     * @param comments
     */
    exports.prototype.withMemo = function (comments) {
        var _this = this;
        _this['comments'] = comments;
        return _this;
    };

    /**
     * Callback function to receive the result of the operation.
     * @callback module:model/refund/Refund~callApiCallback
     * @param {module:exception/OrbipayApiException} exception Error Response, if any.
     * @param {module:model/refund/Refund} data The data returned by the service call.
     */

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/refund/Refund~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.create = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new CreateRefundHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/refund/Refund
     */
    exports.prototype.retrievedBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };


    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/refund/Refund~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.get = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new GetRefundHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    return exports;
}));


