/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/util/CommonUtil'), require('./../constant/HttpParamConstants'));
    }
}(this, function (logger, HttpParamConstants) {
        'use strict';


        /**
         * The CommonUtil model module.
         * @module util/CommonUtil
         * @version 1.0.0
         */

        /**
         * Constructs a new <code>CommonUtil</code>.
         * @alias module:util/CommonUtil
         * @class
         */
        var exports = function () {
        };

        exports.getTimeStamp = function (date) {

            var pad = function (value, width) {
                var padding = '';
                while (padding.length < width - 1 && value < Math.pow(10, width - padding.length - 1))
                    padding += '0';
                return padding + value.toString();
            };

            date = date ? date : new Date();
            var offset = date.getTimezoneOffset();
            return pad(date.getFullYear(), 4)
                + '-' + pad(date.getMonth() + 1, 2)
                + '-' + pad(date.getDate(), 2)
                + ' ' + pad(date.getHours(), 2)
                + ':' + pad(date.getMinutes(), 2)
                + ':' + pad(date.getSeconds(), 2)
                + '.' + pad(date.getMilliseconds(), 3)
                + (offset > 0 ? '-' : '+')
                + pad(Math.floor(Math.abs(offset) / 60), 2)
                + ':' + pad(Math.abs(offset) % 60, 2);
        };

        exports.getXOPAYHeaders = function (attributes) {
            var xOPAYHeaders = '';
            if (attributes && Array.isArray(attributes['list'])) {
                var list = attributes['list'];
                for (var i = 0; i < list.length; i++) {
                    var attribute = list[i];
                    var key = attribute['key'];
                    var value = attribute['value'];
                    if (key !== null && key !== undefined && key !== '' && value !== null && value !== undefined && value !== '') {
                        xOPAYHeaders += key + '=' + value + ';';
                    }
                }
            }
            return xOPAYHeaders.substring(0, xOPAYHeaders.length - 1);
        };

        exports.parseJSON = function (string) {
            var json;
            try {
                json = JSON.parse(string);
            } catch (e) {
                logger.error(e);
            }
            return json;
        };

        exports.maskSensitiveInfo = function (key, value) {
            var maskedValue = value;
            if (HttpParamConstants.SENSITIVE_KEYS.indexOf(key) > -1) {
                if (value && value.length > 5) {
                    maskedValue = '*' + maskedValue.substring(value.length - 4, value.length);
                } else {
                    maskedValue = '****';
                }
            }
            return maskedValue;
        };

        exports.maskSensitiveFormParams = function (logArray) {
            var maskedFormParam = '';
            for (var i = 0; i < logArray.length; i++) {
                var pair = logArray[i];
                var key = pair['key'];
                var maskedValue = pair['value'];
                if (HttpParamConstants.SENSITIVE_KEYS.indexOf(key) > -1) {
                    if (maskedValue && maskedValue.length > 5) {
                        maskedValue = '*' + maskedValue.substring(maskedValue.length - 4, maskedValue.length);
                    } else {
                        maskedValue = '****';
                    }
                }
                maskedFormParam = key + '=' + maskedValue + '&';
            }
            return maskedFormParam.substring(0, maskedFormParam.length - 1);
        };

        exports.getMaskedJsonString = function (object) {
            var maskedString = '';
            try {
                if (object && typeof object === 'object') {
                    maskedString = JSON.stringify(object, exports.maskSensitiveInfo);
                }
            } catch (e) {
                logger.error(e);
            }
            return maskedString;
        };

        exports.getValuesFromMultiMap = function (multiMap, key, isMultipleValuesAllowed) {
            var response;
            if (multiMap && multiMap.map && multiMap.map.hasOwnProperty(key)) {
                if (Array.isArray(multiMap.map[key]) && multiMap.map[key].length > 0) {
                    if (isMultipleValuesAllowed) {
                        response = multiMap.map[key];
                    } else {
                        response = multiMap.map[key][0];
                    }
                } else {
                    multiMap.map.hasOwnProperty(key);
                }
            }
            return response;
        };

        exports.IsJsonString = function (str) {
            try {
                JSON.parse(str);
            } catch (e) {
                return false;
            }
            return true;
        };

        return exports;
    }
));